/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import atomicsets.OrderedLock;
import org.nongnu.savannah.jcurzez.*;

/**
 * A moving window. It also demonstrates how to hide cursor, if possible.
 * 
 * @version 1.0.0
 * @author Thomas Girard
 */

// Two threads are moving this window around.  We should never end up with any funky looking
// non-contiguous window as a result.  Furthermore, no part of the rectangle should not be allowed to 
// move off the screen, and no exceptions (like ArrayOutOfBounds) should be thrown!

public class ThreadedMovingWindow {
	public static void main(String[] args) throws Exception {
		final Screen screen = Screen.getScreen();
		final Rectangle screenSize = screen.getRectangle();
		final int screenWidth = screenSize.getWidth();
		final int screenHeight = screenSize.getHeight();

		screen.clear();
		screen.refresh();
		

		final Window win1 = /*new*/ Window.getWindow(new Rectangle(1, 1, screenWidth / 4 - 1,
				2, new Pen(ForegroundColor.YELLOW, BackgroundColor.BLUE,
						Attribute.BOLD, new OrderedLock()), new Pen(ForegroundColor.RED,
						BackgroundColor.BLUE, Attribute.BOLD, new OrderedLock()), new OrderedLock()));
		final Rectangle winSize = win1.getRectangle();
		final int winWidth = winSize.getWidth();
		final int winHeight = winSize.getHeight();

		win1.printString("Look! I'm moving!");
		win1.setCursorVisibility(Cursor.INVISIBLE);

		Thread horizMover = new Thread() {
			public void run() {
				try {
					while(true){
						win1.move(1, 0);
						win1.touch();
						win1.refresh();
						try {
							Thread.sleep(50);
						} catch (InterruptedException e) {
						}
//						screen.touch();
//						screen.refresh();
					}
				} catch (InconsistentCoordinateException ie) {
					// now go left
				}

				try {
					while(true){
						win1.move(-1, 0);
						win1.touch();
						win1.refresh();
						try {
							Thread.sleep(50);
						} catch (InterruptedException e) {
						}
//						screen.touch();
//						screen.refresh();
					}
				} catch (InconsistentCoordinateException ie) {
					// finished
				}
			}
		};

		Thread diagMover = new Thread() {
			public void run() {
				try {
					while (true) {
						win1.move(1, 1);
						win1.touch();
						win1.refresh();
						try {
							Thread.sleep(50);
						} catch (InterruptedException e) {
						}
//						screen.touch();
//						screen.refresh();
					}
				} catch (InconsistentCoordinateException ie) {
					// now go left
				}

				try {
					while (true) {
						win1.move(-1, -1);
						win1.touch();
						win1.refresh();
						try {
							Thread.sleep(50);
						} catch (InterruptedException e) {
						}
//						screen.touch();
//						screen.refresh();
					}
				} catch (InconsistentCoordinateException ie) {
					// finished
				}
			}
		};
		
		horizMover.start();
		diagMover.start();

	}
}
