/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.peer.win;

import org.nongnu.savannah.jcurzez.peer.ScreenPeer;
import org.nongnu.savannah.jcurzez.*;

/**
 * The <b>win</b> implementation of <b>jcurzez</b>.
 * Almost all methods are native.
 *
 * @version 1.0.0
 * @author Thomas Girard
 */
public final class PeerScreen implements ScreenPeer {
    private static final byte BLACK   = 0;
    private static final byte BLUE    = 1;
    private static final byte GREEN   = 2;
    private static final byte CYAN    = 3;
    private static final byte RED     = 4;
    private static final byte YELLOW  = 5;
    private static final byte MAGENTA = 6;
    private static final byte WHITE   = 7;

    static {
	//If an exception is raised,
	//it isn't caught. Program relying on jcurzez will
	//terminate.
	System.loadLibrary("jcurzez");
    }

    private static native int _begin();

    public PeerScreen() {
	//Initializes curses and colors.
	if (_begin() != 0) {
	    throw new RuntimeException("Unable to initialize win, aborting...");
	}

	ForegroundColor.BLACK.setRealColor(new Byte(BLACK));
	ForegroundColor.BLUE.setRealColor(new Byte(BLUE));
	ForegroundColor.GREEN.setRealColor(new Byte(GREEN));
	ForegroundColor.CYAN.setRealColor(new Byte(CYAN));
	ForegroundColor.RED.setRealColor(new Byte(RED));
	ForegroundColor.YELLOW.setRealColor(new Byte(YELLOW));
	ForegroundColor.MAGENTA.setRealColor(new Byte(MAGENTA));
	ForegroundColor.WHITE.setRealColor(new Byte(WHITE));

	BackgroundColor.BLACK.setRealColor(new Byte(BLACK));
	BackgroundColor.BLUE.setRealColor(new Byte(BLUE));
	BackgroundColor.GREEN.setRealColor(new Byte(GREEN));
	BackgroundColor.CYAN.setRealColor(new Byte(CYAN));
	BackgroundColor.RED.setRealColor(new Byte(RED));
	BackgroundColor.YELLOW.setRealColor(new Byte(YELLOW));
	BackgroundColor.MAGENTA.setRealColor(new Byte(MAGENTA));
	BackgroundColor.WHITE.setRealColor(new Byte(WHITE));
    }

    private static native void _gotoXY(int x, int y);

    /**
     */
    public void gotoXY(int x, int y) {
	_gotoXY(x, y);
    }

    private static native void _printCell(int x, int y,
						       int character,
						       short color,
						       byte attr);

    /**
     */
    public void printCell(int x, int y, final Cell cell) {
	byte fore   =
	    ((Byte) cell.foreground.getRealColor()).byteValue();
	byte back   =
	    ((Byte) cell.background.getRealColor()).byteValue();
	short color = (short) (back * 16 + fore);
	_printCell(x, y,
		   cell.character,
		   color,
		   cell.attribute.getValue());
    }

    private static native int _getColumns();

    /**
     */
    public int getColumns() {
	return _getColumns();
    }

    private static native int _getLines();

    /**
     */
    public int getLines() {
	return _getLines();
    }

    private static native int _getFirstX();

    /**
     */
    public int getFirstX() {
	return _getFirstX();
    }

    private static native int _getFirstY();

    /**
     */
    public int getFirstY() {
	return _getFirstY();
    }

    private static native void _update();

    /**
     */
    public void update() {
	_update();
    }

    private static native void _setCursorVisibility(int v);

    /**
     */
    public void setCursorVisibility(int v) {
	_setCursorVisibility(v);
    }

    private static native void _beep();

    /**
     */
    public void beep() {
	_beep();
    }

    private static native void _end();

    /**
     */
    public void end() {
	_end();
    }
}
