/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import org.nongnu.savannah.jcurzez.*;

/**
 * Tests that <code>System.out</code> and <code>System.err</err>
 * still work as they should.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */

// Not sure if we care about this test -- don't think it really tests our concurrency code.
// Anyway, no line should show up garbled.  Although I believe, for this implementation, we
// allow partial lines to be seen as they print out (to see this set stressConcurrency to true).

public class ThreadedPrintOnStdStreams {
    static volatile boolean done = false;
	
	public static void main(String[] args) {
	//Forces Jcurzez initialization.
	final Screen screen = Screen.getScreen();
	
	
	screen.stressConcurrency = true;
	
	screen.clear();
	
	Thread refresher = new Thread(){
		public void run(){
			while (!ThreadedPrintOnStdStreams.done){
				screen.refresh();
				try{ Thread.sleep(20); } catch (InterruptedException e){}
			}
		}
	};
	refresher.start();
	
	try{Thread.sleep(10);}catch(Exception ex){}
	
	Thread[] children = new Thread[4];
	
	for (int i = 0 ; i < 4 ; i ++){
		final int j = i;
		children[i] =
		new Thread(){
			public void run(){
				System.err.println("This message is printed to System.err\nWith a new line embedded from Thread " + j);
				System.out.println("This message is printed to System.out\nWith a new line embedded from Thread " + j);
//				screen.printString("This message is printed to the jcurzez Screen\nWith a new line embedded from Thread " + j + "\n");
//				try{Thread.sleep((int)(Math.random() * 1000.0));}catch(InterruptedException e){}
				//screen.refresh();
//				screen.printString("Another message is printed to the jcurzez Screen from Thread " + j + "\n");
				//screen.refresh();
			}
		};
		children[i].start();
	}
	
	for (int i = 0 ; i < 4 ; i ++){
		try{children[i].join();}catch(Exception e){}
	}
	
	done = true;
	
	System.out.print("Press a key..."); System.out.flush();
	
	try { System.in.read(); } catch (java.io.IOException ioe) {}
    }
}
